package audio;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class OpenRawEndianness {

    static class RawWaveUtils {

        public static short[] readStream16bit(File file, ByteOrder endianness) throws IOException {
            if (!file.isFile()) {
                return null;
            }

            // lettura del file byte per byte
            InputStream input = new FileInputStream(file);
            byte[] data = new byte[(int) file.length()];
            for (int i = 0; i < file.length(); i++) {
                data[i] = (byte) input.read();
            }

            // conversione dell'array di byte in array di short
            int bufferSize = (int) (file.length() / 2);
            short[] result = new short[bufferSize];
            ByteBuffer.wrap(data).order(endianness).asShortBuffer().get(result);

            return result;
        }
    }
    
    public static void main(String[] args) throws InterruptedException, IOException {

        File fileToOpen = new File("D:/onda_quadra_mono_16.raw");
        short[] shortValues = RawWaveUtils.readStream16bit(fileToOpen, ByteOrder.LITTLE_ENDIAN);
        // stampa a video
        System.out.println("Onda quadra, 16 bit");
        for (short value : shortValues) {
            System.out.println(value + " ");
        }

        // pausa
        BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
        System.out.println("Premere un tasto per continuare...");
        br.readLine();

        fileToOpen = new File("D:/onda_sin_mono_16.raw");
        shortValues = RawWaveUtils.readStream16bit(fileToOpen, ByteOrder.LITTLE_ENDIAN);
        // stampa a video
        System.out.println("Onda sinusoidale, 16 bit");
        for (short value : shortValues) {
            System.out.println(value + " ");
        }

    }
}
